# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

__all__ = [
    "ApplicationStatus",
    "ControllerState",
    "EnabledSubstate",
    "OfflineSubstate",
    "SalIndex",
]

import enum


# MTHexapod and MTRotator have the same enum values for
# ControllerState, OfflineSubstate, and EnabledSubstate.
from .MTRotator import (
    ControllerState,
    EnabledSubstate,
    OfflineSubstate,
)


class ApplicationStatus(enum.IntFlag):
    """Bit masks for the value reported in ``telemetry.application_status``.

    These are from https://github.com/lsst-ts/ts_hexapod_controller/blob/master/include/actuatorTlm.h  # noqa
    """

    FOLLOWING_ERROR = 0x1
    MOVE_COMPLETE = 0x2
    EUI_CONNECTED = 0x4
    RELATIVE_MOVE_MODE = 0x8
    SYNC_MODE = 0x10
    COMMAND_REJECTED = 0x20
    SAFETY_INTERLOCK = 0x40
    EXTEND_LIMIT_SWITCH = 0x80
    RETRACT_LIMIT_SWITCH = 0x100
    ETHERCAT_PROBLEM = 0x200
    DDS_COMMAND_SOURCE = 0x400
    """1=DDS is commanding, 0=EUI is commanding"""

    MOTION_TIMEOUT = 0x800
    """Motion timed out: one or more struts not settling
    or command values did not change.
    """

    DDS_CONNECTED = 0x1000
    DRIVE_FAULT = 0x2000
    SIMULINK_FAULT = 0x4000
    LUT_TABLE_INVALID = 0x8000


class SalIndex(enum.IntEnum):
    CAMERA_HEXAPOD = 1
    M2_HEXAPOD = 2



# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

__all__ = ["ControllerState", "OfflineSubstate", "EnabledSubstate", "ApplicationStatus"]

import enum


class ControllerState(enum.IntEnum):
    """Controller state reported as ``telemetry.state``.

    The names and meanings are the same as SAL summary states
    but the numeric values are different.

    Value is ``telemetry.state``.

    Called ``States`` in Moog code.
    """

    STANDBY = 0
    DISABLED = enum.auto()
    ENABLED = enum.auto()
    OFFLINE = enum.auto()
    FAULT = enum.auto()


class OfflineSubstate(enum.IntEnum):
    """Controller substate for the OFFLINE state.

    Value reported in ``telemetry.offline_substate``.

    This is enum ``OfflineSubStates`` in Moog code.
    """

    PUBLISH_ONLY = 0
    AVAILABLE = enum.auto()


class EnabledSubstate(enum.IntEnum):
    """Controller substate for the ENABLED state.

    Value reported in ``telemetry.enabled_substate``.

    This is enum ``EnabledSubStates`` in Moog code.
    """

    STATIONARY = 0
    MOVING_POINT_TO_POINT = enum.auto()
    SLEWING_OR_TRACKING = enum.auto()
    CONTROLLED_STOPPING = enum.auto()
    INITIALIZING = enum.auto()
    RELATIVE = enum.auto()
    CONSTANT_VELOCITY = enum.auto()


class ApplicationStatus(enum.IntFlag):
    """Bit masks for the value reported in ``telemetry.application_status``.

    These are from https://github.com/lsst-ts/ts_rotator_controller/blob/master/include/actuatorTlm.h  # noqa
    """

    EUI_CONNECTED = 0x4
    COMMAND_REJECTED = 0x20
    SAFETY_INTERLOCK = 0x40
    EXTEND_LIMIT_SWITCH = 0x80
    RETRACT_LIMIT_SWITCH = 0x100
    ETHERCAT_PROBLEM = 0x200
    DDS_COMMAND_SOURCE = 0x400
    DDS_CONNECTED = 0x1000
    DRIVE_FAULT = 0x2000
    SIMULINK_FAULT = 0x4000
    ENCODER_FAULT = 0x8000



# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
"""WARNING:

As of 2019-10-31 these enumeration values match the values actually output
by the ATPneumatics CSC, but they DO NOT MATCH the enumeration values in
ATPneumatics_Events.xml in ts_xml. The XML file is missing the unwanted
summary states (DISABLED through STANDBY).

Our long range plan is to change the CSC (and these enums) to match
ATPneumatics_Events.xml in ts_xml.
"""

__all__ = ["MirrorCoverState", "CellVentState", "AirValveState"]

import enum


class MirrorCoverState(enum.IntEnum):
    DISABLED = 1
    ENABLED = 2
    FAULT = 3
    OFFLINE = 4
    STANDBY = 5
    CLOSED = 6
    OPENED = 7
    INMOTION = 8
    INVALID = 9


class CellVentState(enum.IntEnum):
    DISABLED = 1
    ENABLED = 2
    FAULT = 3
    OFFLINE = 4
    STANDBY = 5
    OPENED = 6
    CLOSED = 7
    INMOTION = 8


class AirValveState(enum.IntEnum):
    DISABLED = 1
    ENABLED = 2
    FAULT = 3
    OFFLINE = 4
    STANDBY = 5
    OPENED = 6
    CLOSED = 7


class VentsPosition(enum.IntEnum):
    OPENED = 0
    CLOSED = 1
    PARTIALLYOPENED = 2



# This file is part of ts_idl.
#
# Developed for Vera Rubin Observatory.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License

__all__ = ["Mode", "FilterType"]

import enum


class Mode(enum.IntEnum):
    """MTAOS wavefront sensing mode.

    1 - LSST Camera full array mode.
    2 - LSST Camera corner wavefront sensors.
    3 - ComCam full array mode

    """

    LsstCamFAM = 1
    LsstCamCWS = enum.auto()
    ComCam = enum.auto()


class FilterType(enum.IntEnum):
    u = 1
    g = enum.auto()
    r = enum.auto()
    i = enum.auto()
    z = enum.auto()
    y = enum.auto()
    ref = enum.auto()



"""Sphinx configuration file for TSSW package"""

from documenteer.conf.pipelinespkg import *  # noqa
import lsst.ts.idl  # noqa

project = "ts_idl"
html_theme_options["logotext"] = project  # noqa
html_title = project
html_short_title = project
doxylink = {}  # Avoid warning: Could not find tag file _doxygen/doxygen.tag

intersphinx_mapping["ts_xml"] = ("https://ts-xml.lsst.io", None)  # noqa



# This file is part of ts_idl.
#
# Developed for the Vera C. Rubin Observatory Telescope and Site Systems.
# This product includes software developed by the LSST Project
# (https://www.lsst.org).
# See the COPYRIGHT file at the top-level directory of this distribution
# for details of code ownership.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

__all__ = ["DeviceId", "DEVICE_GROUPS", "DEVICE_GROUP_IDS"]

from enum import IntEnum


class DeviceId(IntEnum):
    bombaAguaFriaP01 = 1
    generalP01 = 2
    valvulaP01 = 3
    chiller01P01 = 101
    chiller02P01 = 102
    chiller03P01 = 103
    crack01P02 = 201
    crack02P02 = 202
    fancoil01P02 = 301
    fancoil02P02 = 302
    fancoil03P02 = 303
    fancoil04P02 = 304
    fancoil05P02 = 305
    fancoil06P02 = 306
    fancoil07P02 = 307
    fancoil08P02 = 308
    fancoil09P02 = 309
    fancoil10P02 = 310
    fancoil11P02 = 311
    fancoil12P02 = 312
    manejadoraLower01P05 = 401
    manejadoraLower02P05 = 402
    manejadoraLower03P05 = 403
    manejadoraLower04P05 = 404
    manejadoraSblancaP04 = 501
    manejadoraSlimpiaP04 = 502
    vea01P05 = 601
    vea08P05 = 602
    vea09P05 = 603
    vea10P05 = 604
    vea11P05 = 605
    vea12P05 = 606
    vea13P05 = 607
    vea14P05 = 608
    vea15P05 = 609
    vea16P05 = 610
    vea17P05 = 611
    vea01P01 = 701
    vec01P01 = 702
    vin01P01 = 703
    vex03LowerP04 = 801
    vex04CargaP04 = 802


# Dict of index: DeviceId, where index is the index of the device in DeviceId.
# Used to understand the bits in the device_ids field of the deviceEnabled event.
DeviceIndex = {i: dev_id for i, dev_id in enumerate(DeviceId)}

# Dict grouping MQTT topics representing HVAC devices together.
DEVICE_GROUPS = {
    "CHILLER": [
        "LSST/PISO01/CHILLER_01",
        "LSST/PISO01/CHILLER_02",
        "LSST/PISO01/CHILLER_03",
    ],
    "CRACK": [
        "LSST/PISO02/CRACK01",
        "LSST/PISO02/CRACK02",
    ],
    "FANCOIL": [
        "LSST/PISO02/FANCOIL01",
        "LSST/PISO02/FANCOIL02",
        "LSST/PISO02/FANCOIL03",
        "LSST/PISO02/FANCOIL04",
        "LSST/PISO02/FANCOIL05",
        "LSST/PISO02/FANCOIL06",
        "LSST/PISO02/FANCOIL07",
        "LSST/PISO02/FANCOIL08",
        "LSST/PISO02/FANCOIL09",
        "LSST/PISO02/FANCOIL10",
        "LSST/PISO02/FANCOIL11",
        "LSST/PISO02/FANCOIL12",
    ],
    "MANEJADORA_LOWER": [
        "LSST/PISO05/MANEJADORA/LOWER_01",
        "LSST/PISO05/MANEJADORA/LOWER_02",
        "LSST/PISO05/MANEJADORA/LOWER_03",
        "LSST/PISO05/MANEJADORA/LOWER_04",
    ],
    "MANEJADORA": [
        "LSST/PISO04/MANEJADORA/GENERAL/SBLANCA",
        "LSST/PISO04/MANEJADORA/GENERAL/SLIMPIA",
    ],
    "VEA": [
        "LSST/PISO05/VEA_01",
        "LSST/PISO05/VEA_08",
        "LSST/PISO05/VEA_09",
        "LSST/PISO05/VEA_10",
        "LSST/PISO05/VEA_11",
        "LSST/PISO05/VEA_12",
        "LSST/PISO05/VEA_13",
        "LSST/PISO05/VEA_14",
        "LSST/PISO05/VEA_15",
        "LSST/PISO05/VEA_16",
        "LSST/PISO05/VEA_17",
    ],
    "SALA_MAQUINAS": [
        "LSST/PISO01/VEA_01",
        "LSST/PISO01/VIN_01",
        "LSST/PISO01/VEC_01",
    ],
    "VEX": [
        "LSST/PISO04/VEX_03/DAMPER_LOWER/GENERAL",
        "LSST/PISO04/VEX_04/ZONA_CARGA/GENERAL",
    ],
}

# Dict assigning an ID to each device group. This is used in dictionary
# comprehension so it is better to keep it as a dict instead of an enum.
DEVICE_GROUP_IDS = {
    "CHILLER": 100,
    "CRACK": 200,
    "FANCOIL": 300,
    "MANEJADORA_LOWER": 400,
    "MANEJADORA": 500,
    "VEA": 600,
    "SALA_MAQUINAS": 700,
    "VEX": 800,
}
